﻿Option Strict On

Imports System.Math

Public Module ODE
    Public Delegate Function MathFunction(ByVal x As Double) As Double
    Public Delegate Function Derivative(ByVal x As Double, _
                                        ByVal y As Double) As Double

    ' Euler-Verfahren
    Public Function Euler(ByVal df As Derivative, ByVal x As Double, _
                          ByVal y As Double, ByVal h As Double) As Double
        Return y + df(x, y) * h
    End Function

    ' Runge-Kutta-Verfahren (RK4)
    Public Function RK4(ByVal df As Derivative, ByVal x As Double, _
                        ByVal y As Double, ByVal h As Double) As Double

        Dim k1 = df(x, y)
        Dim k2 = df(x + h / 2, y + h * k1 / 2)
        Dim k3 = df(x + h / 2, y + h * k2 / 2)
        Dim k4 = df(x + h, y + h * k3)

        Return y + h * (k1 + 2 * k2 + 2 * k3 + k4) / 6
    End Function
End Module
